﻿using System;
using System.Data;
using System.Linq;
using System.Windows.Forms;

namespace LINQToDataSet.LinqDataView
{
    public partial class MainForm : Form
    {
        public MainForm( )
        {
            InitializeComponent( );

            LoadData( );

            dataGridView1.DataSource = shippingDS1.SalesOrderHeader;
        }

        private void LoadData( )
        {
            ShippingDSTableAdapters.AddressTableAdapter addressLoader = new ShippingDSTableAdapters.AddressTableAdapter( );
            ShippingDSTableAdapters.SalesOrderHeaderTableAdapter headerLoader = new ShippingDSTableAdapters.SalesOrderHeaderTableAdapter( );

            addressLoader.Fill( shippingDS1.Address );
            headerLoader.Fill( shippingDS1.SalesOrderHeader );

            ZipDataTableAdapters.ZipLocationTableAdapter zipLoader = new ZipDataTableAdapters.ZipLocationTableAdapter( );

            zipLoader.Fill( zipData1.ZipLocation );
        }

        private void button1_Click( object sender, EventArgs e )
        {

            var query = from row in shippingDS1.Tables[ "SalesOrderHeader" ].AsEnumerable( )
                        orderby Distance( zipTB.Text, row.GetParentRow( "ShipToAddress" )[ "PostalCode" ].ToString( ) ) descending
                        select row;

            dataGridView1.DataSource = query.AsDataView( );
        }

        private double Distance( string zip1, string zip2 )
        {
            double lat1; double long1; double lat2; double long2;

            var zipLoc1 = zipData1.ZipLocation.FindByZip( zip1 );
            var zipLoc2 = zipData1.ZipLocation.FindByZip( zip2 );

            lat1 = ( double )zipLoc1.Latitude;
            long1 = ( double )zipLoc1.Longitude;

            if ( zipLoc2 == null ) return double.MaxValue;

            lat2 = ( double )zipLoc2.Latitude;
            long2 = ( double )zipLoc2.Longitude;

            Func<double, double> degreeToRadian = deg => deg * Math.PI / 180;
            Func<double, double> radianToDegree = rad => rad / Math.PI * 180;

            double theta = long1 - long2;
            double dist = Math.Sin( degreeToRadian( lat1 ) )
                * Math.Sin( degreeToRadian( lat2 ) )
                + Math.Cos( degreeToRadian( lat1 ) )
                * Math.Cos( degreeToRadian( lat2 ) )
                * Math.Cos( degreeToRadian( theta ) );
            dist = Math.Acos( dist );
            dist = radianToDegree( dist );
            dist = dist * 60 * 1.1515;

            return dist;
        }
    }
}